#!/usr/bin/perl -w

#
#  COPYRIGHT_COMMENT_TAG_BEGIN
#
#  Copyright IBM Corp. 2006
#  The source code for this program is not published or otherwise
#  divested of its trade secrets, irrespective of what has
#  been deposited with the U.S. Copyright Office.
#
#  COPYRIGHT_COMMENT_TAG_END
#
# %Z%%M%    %I%  %W%  %G%  %U%

use strict;
use Getopt::Long;
use Term::ANSIColor;
use Cwd 'abs_path';

# Return an array of the name of all cec on the HMC
sub get_hmc_cec {
        my ($user, $hmc) = @_;
	my @cec = ();
        my $command = "/usr/bin/ssh $user\@$hmc lssyscfg -r sys -F name 2>/dev/null";
        my @output = `LANG=C $command`;
        foreach my $output (@output) {
                chomp($output);
		push(@cec, $output);
        }

        return @cec; 
}

# Return a hash of partition information with lpar_name
sub get_hmc_lpars {
        my ($cec, $hmc_user, $hmc) = @_;
        my %hash = ();
	my $split_value = ",";
        my $command = "/usr/bin/ssh $hmc_user\@$hmc lssyscfg -r lpar -m \"$cec\" -F name,lpar_id 2>/dev/null";

        my @output = `LANG=C $command`;
	foreach my $output (@output) {
		chomp(@output);
                my $count = @output;
                if ( ( $output ne "" ) &&
                     ( $output !~ /No results were found./ ) &&
                     ( $output !~ /FullSystemPartition/ ) ) {
                	my ($name, $lpar_id) = split (/$split_value/, $output);
                        my %lpar = (
                                "lpar_id" => $lpar_id,
                        );

                        $hash{$name} = \%lpar;
                }
        }

        return %hash;
}

# Return a hash of HMC partition information with partition IPs.
sub get_lspartition {
        my ($hmc_user, $hmc) = @_;
        my %hash;

		my $split_delimiter = GetSplitValue($hmc_user, $hmc);
        my $command = "/usr/bin/ssh $hmc_user\@$hmc lspartition -ix 2>/dev/null";
        my @output = `LANG=C $command`;
        foreach my $output (@output) {
                chomp($output);
                my @partition = split(/;/, $output);
                foreach my $partition (@partition) {
                	my ($lpar_id, $address, $active, $hostname, $OStype, $OSlevel) = split(/$split_delimiter/, $partition);
                        my $name = $output;
                        $hash{$lpar_id}{"address"}  = $address;
                        $hash{$lpar_id}{"active"}   = $active;
                        $hash{$lpar_id}{"hostname"} = $hostname;
                        $hash{$lpar_id}{"OStype"}   = $OStype;
                        $hash{$lpar_id}{"OSlevel"}  = $OSlevel;
                }
        }

        return %hash;
}

################################################################
# Delimiter of lspartition is different between the HMC versions
# For HMC version greater than 7.3.0 , the delimiter is "'"
# For HMC version less than 7.3.4, it is ":"
###############################################################
sub GetSplitValue {
        my ($hmc_user, $hmc) = @_;
        my $command = "/usr/bin/ssh $hmc_user\@$hmc lshmc -V 2>/dev/null";
        my @output = `LANG=C $command`;
        my $version = '';
        my $release = '';
        foreach my $output (@output)
        {
                chomp($output);
                if ( $output =~ /Version:(.*)/ ){
                $version = $1;
                }
                if ( $output =~ /Release:(.*)/ ){
                $release = $1;
                my ($tmp1,$tmp2) = split(/\./,$release);
                $release=$tmp1.".".$tmp2;
                }
        }
        if ( $version >= 7 && $release > 3.3 ){
        return ","; }
        else {
        return ":"; }
}


# Determine if SSH is installed
# Return 0 if ssh is installed
sub has_ssh {
	my $rc = 1;
        my $command = "/usr/bin/lslpp -L openssh.base.client 2>/dev/null";
        my @output = `LANG=C $command`;
        if ( $? == 0 ) {
                $rc = 0;
        }

        return $rc;
}

# Determine if CEC agent is installed
# Return 0 if CEC agent is installed
sub has_cec_agent {
	my $rc = 1;
	my $command = "/usr/bin/lslpp -L itm.base.cec.agent 2>/dev/null";
	my @output = `LANG=C $command`;
        if ( $? == 0 ) {
                $rc = 0;
        }

        return $rc;
}

# Return 0 if key exists for the server
sub has_key {
        my ($user) = @_;
        my $rc = 1;

        # Determine if ssh keys has been generated
	my $file = "/.ssh/id_rsa.pub";
	my $key  = "/.ssh/id_rsa.pub";

	if ($user ne "root") {
		$file = "/home/$user/$file";
		$key  = "/home/$user/$key";
	}
	 
        if ( ( (-e $file) && !(-z $file) ) || 
	     ( (-e $key) && !(-z $key) ) ) {
                $rc = 0;
        }

        return $rc;
}

# Generate key for the host
# Return 0 if key generation is successful
sub gen_key {
        my ($user, $hmc_user, $hmc, $force, $cat) = @_;

        my $file = "//.ssh/id_rsa";
        my $key  = "/.ssh/id_rsa.pub";
        my $rc = 1;
	my $command;

        if ($user ne "root") {
                $file = "/home/$user/.ssh/id_rsa";
                $key  = "/home/$user/.ssh/id_rsa.pub";
        }

	if ($force eq "1") {
		if ( -e $key ) {
			$command = "/usr/bin/rm $key";
			`LANG=C $command`;
			if ($? != 0) { return $rc; }
		}

		if ( -e $file ) {
			$command = "/usr/bin/rm $file";
			`LANG=C $command`;
                	if ($? != 0) { return $rc; }
		}
	}

        if (has_key($user) != 0) {
        	$command = "/usr/bin/ssh-keygen -t rsa -f $file -N \"\"";
		system("LANG=C $command");
	}

	# cat the key only for VIOS testing - hidden flag
        if ($cat eq "1") {
		$command = "/usr/bin/cat $key";
                my @output = `LANG=C $command`;
		print "@output\n";
		return 0;
       	}

        scp_key($hmc_user, $hmc, $key);
        if ($? == 0) { $rc = 0; }

        return $rc;
}

# Return 0 if key copying to HMC is successful
sub scp_key {
        my ($hmc_user, $hmc, $key) = @_;
        my $file = sprintf("%s/%s_%d", "/tmp", "ssh_key", $$);
	my $rc = 1;

	print "Setting up credential with $hmc.\n";
        my $command = "/usr/bin/scp -q $hmc_user\@$hmc:.ssh/authorized_keys2 $file";
	system("LANG=C $command");
        if ($? == 0) {
		$command = "/usr/bin/cat $key >> $file";
		`LANG=C $command`;
                $command = "/usr/bin/scp -q $file $hmc_user\@$hmc:.ssh/authorized_keys2";
		system("LANG=C $command");
		$command = "/usr/bin/rm $file 2>/dev/null";
		`LANG=C $command`;
		$rc = $?;
        }

        return $rc;
}

# Prompt for HMC information for key setup.
sub prompt {
	my ($file, $user, $hmc, $force, $cat) = @_;
	my $manage = "";

	if ($hmc eq "") {
		print "Enter the name of the HMC server: ";
	} else {
		print "Enter the name of the HMC server[$hmc]: ";
	}

	chomp(my $value = (<>));
	$hmc = (length($value) != 0) ? $value : $hmc; 


	my $hmc_user = "hscroot";
	print "Enter the name of the HMC user[$hmc_user]: ";
        chomp($value = (<>));
        $hmc_user = (length($value) != 0) ? $value : $hmc_user;

        gen_key($user, $hmc_user, $hmc, $force, $cat);

	# Ask if user would like to set managed system
	until ( ($value eq "yes") || ($value eq "y") ) {
		print "Would you like to continue setting a default managed system [yes]/no? ";
		chomp($value = (<>));
		if (length($value) == 0) { $value = "yes"; }
		$value = lc($value);
		if ( ($value eq "no") || ($value eq "n") ) { 
			last; 
		}
	}

	if ( ($value eq "yes") || ($value eq "y") ) {	
		my $hostid = "";
		my @cec = get_hmc_cec($hmc_user, $hmc);
		if (! @cec) {
			print "No managed system found.\n";
		}

		chomp(my $hostname = `/usr/bin/hostname`);
		chomp(my $host     = `/usr/bin/host $hostname`);
		my ($nulll, $null2, $hostip) = split(/\s+/, $host);
		my %lspartition = get_lspartition($hmc_user, $hmc);
		foreach my $partition_id (keys %lspartition) {
			if ($lspartition{$partition_id}{"address"} eq $hostip) {
				$hostid = $partition_id;
				last;
			}
		}

		if ($hostid ne "") {
			my $proceed = "true";
			foreach my $cec (@cec) {
				my %lpar = get_hmc_lpars($cec, $hmc_user, $hmc);
				foreach my $lpar_name (keys %lpar) {
					my $lpar_id = $lpar{$lpar_name}{"lpar_id"};
					if ($lpar_id eq $hostid) {
						$manage = $cec;
						$proceed = "false";
						last;
					}
				}

				if ($proceed eq "false") {
					last;
				}
			}
		}
	
		if (@cec) {
			my $count = @cec;
			my $exit = $count + 1;
			my $value = "";
			until ($value eq $exit) {
				print "Enter or select the managed system: \n";

				for (my $i = 0; $i < $count; $i++) {
					my $index = $i + 1;
					if ($cec[$i] eq $manage) {
						print "$index : $cec[$i] [default]\n";
					} else {
						print "$index : $cec[$i]\n";
					}
				}
				print "$exit : quit\n";

				chomp($value = (<>));
				if ( ($value > 0) && ($value < $exit) ) {
					$manage = $cec[$value - 1];
					last;
				}
			}
		}
	}

	write_info($file, $hmc, $manage, $hmc_user);
}

# Write HMC/Manage system information into a file for CEC agent to query
sub write_info {
	my ($file, $hmc, $manage, $hmc_user) = @_;

	open(INFO, ">$file"); 
	print INFO "HMC:$hmc\n";
	print INFO "MANAGE:$manage\n";
	print INFO "HMC_USER:$hmc_user\n";
	close(INFO);
}

# Return the content of a file containing HMC/Manage System information
# for CEC agent
sub query_info {
	my ($user, $file) = @_;
	my $rc = 0;

        if ( (-e $file) && !(-z $file) ) {
                my $command = "/usr/bin/cat $file\n";
                my @output = `LANG=C $command`;
                print @output;
        } else {
                print "Information file $file does not exist.\n";
                print "Please setup credential key to populate the file.\n";
                $rc = 1;
        }

	return $rc;
}

# Command usage
sub usage {

        print color("bold"), "Purpose", color("reset"),
              "\n\tSet up SSH credential between the executing machine to an HMC with the default hscroot user.\n\n",
              color("bold"), "Syntax", color("reset"),
              "\n\t$0 [-q] | [ [-f] -s <hmc> [-u <user>] [-m <manage>] ] | [-h]\n\n",
              color("bold"), "Flags", color("reset"),
              "\n\t-q\t\tQuery for current HMC and Managed System setup.\n",
	      "\t-u <user>\tSpecify the HMC user.\n",
              "\t-s <hmc>\tSpecify the HMC to setup SSH credential with.\n",
              "\t-m <manage>\tSpecify the managed system to be logged.\n",
              "\t-f\t\tForce the regeneration of key setup.\n",
              "\t-h\t\tPrint this usage.\n";

	exit(1);
}

###################################################
#                      MAIN                       #
###################################################
my ($hmc, $force, $help, $query, $manage, $cat, $hmc_user) = undef;
my $retval = GetOptions("s:s"    => \$hmc,
                        "c"      => \$cat,
                        "f"      => \$force,
                        "m:s"    => \$manage,
			"q"	 => \$query,
			"u"      => \$hmc_user,
                        "help|h|\?" => \$help
);

if ( $help || 
     ( $force && !defined($hmc) ) || 
     ( $query && (defined($manage) || defined($hmc) || $force ) ) ||
     ( defined($hmc) && ($hmc eq "") ) ||
     ( defined($manage) && ($manage eq "") )
   ) {
	usage();
}

# Create Config path
my $candle_home = "";
my $key_path = "";
#Get the absolute path of key.pl. eg: "/opt/IBM/ITM/aix513/ph/bin/key.pl"
$key_path = abs_path($0);

# Look for "/aix51?/p?/bin/key.pl"
if ($key_path =~ m#/aix51(3|6)/p(h|k)/bin/key.pl#) 
{
   # get the portion of the string before the match.
   $candle_home = $`;
}
else
{
}

if ($candle_home eq "") 
{
   print "Enter the name of the ITM Tivoli Monitoring directory: ";
   chomp(my $value = (<>));
   $candle_home = (length($value) != 0) ? $value : $candle_home;
}

# Remove trailing / if any.
$candle_home =~ s/\/$//;
#print "key.pl configuration file will be stored in $candle_home/config\n";
#
my $file = "$candle_home/config/key.pl_managefile";
my $hmc_server = "";
chomp(my $user = `/usr/bin/whoami 2>/dev/null`);
chomp(my $hmc_ip = `LANG=C /usr/bin/lsrsrc -dx IBM.ManagementServer Hostname ManagerType | /usr/bin/grep -w HMC | /usr/bin/head -1 | cut -d: -f1 | sed \'s/\"//g\' 2>/dev/null`); 

if ( $? == 0) { 
	($hmc_server) = split(/\s+/, `/usr/bin/host $hmc_ip 2>/dev/null`);          
}

if (!defined($hmc_user)) { $hmc_user = "hscroot"; }
if (!defined($manage))   { $manage = ""; }

$force = (defined($force)) ? "1" : "0";
$cat   = (defined($cat))   ? "1" : "0";

# Already has SSH installed
if (has_ssh() == 0) {
	if ( !defined($hmc) && ($manage eq "") && (! $query) ) {
		prompt($file, $user, $hmc_server, $force, $cat);
	} else {
		if ($query) {
			query_info($user, $file);
		} else {
			gen_key($user, $hmc_user, $hmc, $force, $cat);
			write_info($file, $hmc, $manage, $hmc_user);
		}
	}
} else {
	print "Please install the openssh.base.client fileset.";
}
